const fs = require('fs')
const path = require('path')
const crypto = require('crypto')
const { Random, lodash, getAppByCtx, getConfig } = require('multiple-quick-koa')
const Captchapng = require('captchapng')
const qr = require('qr-image')
const OSS = require('ali-oss')
const axios = require('axios')
const util = require('util')
const jwt = require('jsonwebtoken')

const Email = require('../../services/email')
const Geo = require('../../services/geo')

const annotationsReg = /(?:^|\n|\r)\s*\/\*[\s\S]*?\*\/\s*(?:\r|\n|$)/g
const apiReg = /@api(Name|Desc|Param|Route|Return)(\s+)(.*)\n/g

/**
 * @apiRoute upload
 * @apiName 上传
 * @apiParam subject/string/付款名称
 * @apiParam total_fee/string/支付金额
 * @apiReturn {url}
 */

exports.upload = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)

  if (!(ctx.request.files && ctx.request.files.file)) {
    throw new Error('请选择上传文件')
  }

  const { file, option = 'url' } = ctx.request.files

  const appConfig = getConfig(app)

  const { site_host } = await appConfig.getObject('base')

  const reader = fs.readFileSync(file.path)

  const fsHash = crypto.createHash('md5')
  fsHash.update(reader)
  const md5 = fsHash.digest('hex')
  const newFilename = `${md5}.${
    file.name.split('.')[file.name.split('.').length - 1]
  }`

  const imgData = Buffer.from(reader).toString('base64')
  // 转换为 data:image/jpeg;base64,***** 格式的字符串
  const base64 = `data:${file.name.split('.')[1]};base64,${imgData}`

  // const { dir, upload } = app.config.static;
  const targetPath = `${path.resolve(
    __dirname,
    `${process.cwd()}/public/upload`
  )}/${newFilename}`

  const isExists = fs.existsSync(targetPath)
  const responseData = lodash.pick(
    {
      url: `//${site_host || 'api.kuashou.com'}/upload/${newFilename}`,
      base64,
      card: imgData,
    },
    [option]
  )
  if (isExists) return ctx.SUCCESS(responseData)

  fs.writeFileSync(targetPath, reader)
  return ctx.SUCCESS(responseData)
}

exports.oss_upload = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  if (
    !(ctx.request.files && ctx.request.files.file) &&
    !(ctx.request.body.url && ctx.request.body.filename)
  ) {
    ctx.status = 500
    ctx.ERROR('请选择上传文件')
  }

  const appConfig = getConfig(app)

  const {
    accessKeyId,
    accessKeySecret,
    bucket,
    region,
    cdn_host,
    oss_host,
  } = await appConfig.getObject('oss')

  if (!(accessKeyId && accessKeySecret && bucket && region)) {
    throw new Error('没有配置oss设置')
  }

  const client = new OSS({
    accessKeyId,
    accessKeySecret,
    bucket,
    region,
    secure: true,
  })
  let fileName
  let imageData
  if (ctx.request.files && ctx.request.files.file) {
    const { file } = ctx.request.files
    fileName = `${
      String(new Date().getTime()).split('').reverse().join('') + Math.random()
    }.${file.name.split('.')[file.name.split('.').length - 1]}`
    imageData = file.path
  } else {
    const { url: imageUrl, filename: imageFileName } = ctx.request.body
    fileName = imageFileName
    const formatUrl = imageUrl.startsWith('//') ? 'http:' + imageUrl : imageUrl
    try {
      imageData = await axios
        .get(formatUrl, {
          responseType: 'arraybuffer',
        })
        .then((res) => res.data)
    } catch (e) {
      ctx.status = 500
      ctx.ERROR('请求图片错误')
    }
  }
  const result = await client.put(fileName, imageData)
  if (result.res && result.res.status === 200) {
    if (oss_host && cdn_host) {
      const url = result.url.replace(oss_host, cdn_host)
      return ctx.SUCCESS({
        url,
      })
    } else {
      const url = result.url
      return ctx.SUCCESS({
        url,
      })
    }
  } else {
    throw new Error(result.res.statusMessage)
  }
}

exports.captcha = async (ctx) => {
  const number = Random.string('number', 4)
  const p = new Captchapng(80, 30, number) // width,height,numeric captcha
  p.color(0, 0, 0, 0) // First color: background (red, green, blue, alpha)
  p.color(80, 80, 80, 255) // Second color: paint (red, green, blue, alpha)

  const img = p.getBase64()
  ctx.SUCCESS({
    img: `data:image/png;base64,${img}`,
    result: number,
  })
}

exports.email = async (ctx) => {
  const email = new Email({
    email: '40854@qq.com',
    password: 'xszvtgcjjqogcahb',
  })

  const { to, subject, text, html } = ctx.request.body

  await email.send({
    to,
    subject,
    text,
    html,
  })

  return ctx.SUCCESS()
}

exports.createQr = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)

  const { url } = ctx.request.query
  const img = qr.image(url, {
    size: 10,
  })
  ctx.type = 'image/png'
  ctx.body = img
}

exports.channel = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)

  const channelId = ctx.query.channelId
  if (ctx.ws) {
    app.ws[channelId] = await ctx.ws()

    const send = (data) => app.ws[channelId].send(JSON.stringify(data))
    const sendMessage = (content) => {
      console.log('收到信息了')
      const { type, payload } = content
      send({
        type,
        payload: payload.message,
      })
    }

    app.event.removeAllListeners()
    app.event.on(channelId, sendMessage)
  } else {
    throw new Error('这是一个websocket url')
  }
}

exports.swagger = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)
  if (app.cache && app.cache.get('swagger'))
    return ctx.SUCCESS(app.cache.get('swagger'))

  const filePath = path.resolve(
    __dirname,
    `${process.cwd()}/app/${appName}/plugins`
  )
  const apis = []
  const pathList = fs.readdirSync(filePath)
  for (let i = 0; i < pathList.length; i++) {
    const folder = pathList[i]
    const isFolder = fs.lstatSync(path.resolve(filePath, folder)).isDirectory()
    if (isFolder) {
      const models = []
      if (app.model[folder]) {
        const result = await app.sequelize
          .getQueryInterface()
          .describeTable(folder)
        Object.keys(result).forEach((attr) => {
          models.push(lodash.merge(app.attributes[folder][attr], result[attr]))
          // models[attr] = lodash.merge(app.attributes[folder][attr], result[attr]);
        })
      }

      const controllers = []
      const controllerFilePath = path.resolve(filePath, folder, 'controller.js')
      const isControllerExist = fs.existsSync(controllerFilePath)
      if (isControllerExist) {
        const controllerFile = fs.readFileSync(controllerFilePath, 'utf-8')
        const annotations = controllerFile.match(annotationsReg)

        if (annotations) {
          for (let j = 0; j < annotations.length; j++) {
            const annotation = annotations[j]
            let res
            const obj = {}
            while ((res = apiReg.exec(annotation))) {
              res = res.map((item) => item.toLowerCase())
              if (res[1] === 'param') {
                const arr = res[3].split('/')
                if (obj[res[1]]) {
                  obj[res[1]].push({
                    key: arr[0],
                    type: arr[1],
                    name: arr[2],
                  })
                } else {
                  obj[res[1]] = []

                  obj[res[1]].push({
                    key: arr[0],
                    type: arr[1],
                    name: arr[2],
                  })
                }
              } else {
                obj[res[1]] = res[3]
              }
            }
            controllers.push(obj)
          }
        }
      }
      apis.push({
        name: folder,
        desc: lodash.get(app, `config.${folder}.name`, `${folder}`),
        models,
        controllers,
      })
    }
  }
  app.cache && app.cache.set('swagger', apis)
  // console.log(dir)
  ctx.SUCCESS(apis)
}

exports.getLocation = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { provinceName, cityName, address } = ctx.request.body
  if (!provinceName || !cityName || !address) {
    return ctx.SUCCESS({
      longitude: null,
      latitude: null,
    })
  }
  const appConfig = getConfig(app)
  const { tencent_key } = await appConfig.getObject('map')

  const geo = new Geo(tencent_key)
  const addressName = address || `${provinceName}${cityName}`
  const result = await geo.getToLngAndLat(addressName)

  const { lng: longitude, lat: latitude } = result.location

  return ctx.SUCCESS({
    longitude,
    latitude,
  })
}

exports.getLngAndLat = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { longitude, latitude } = ctx.request.body

  const appConfig = getConfig(app)
  const { tencent_key } = await appConfig.getObject('map')

  const geo = new Geo(tencent_key)
  const res = await geo.getToLoation({
    longitude,
    latitude,
  })

  return ctx.SUCCESS(res)
}

exports.syncModel = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { model: _model } = ctx.request.body

  let modellist = []
  if (_model === 'all') {
    modellist = Object.keys(app.model)
  } else {
    modellist = [_model]
  }

  for (let index = 0; index < modellist.length; index++) {
    const model = modellist[index]
    try {
      const list = await app.model[model].findAll({
        raw: true,
      })

      const result = await app.sequelize
        .getQueryInterface()
        .describeTable(model)

      const idType = result.id.type

      if (idType.includes('TINYINT')) {
        if (idType.includes('UNSIGNED') && list.length >= 128) {
          throw new Error('id超过类型')
        } else if (!idType.includes('UNSIGNED') && list.length >= 256) {
          throw new Error('id超过类型')
        }
      } else if (idType.includes('SMALLINT')) {
        if (idType.includes('UNSIGNED') && list.length >= 32767) {
          throw new Error('id超过类型')
        } else if (!idType.includes('UNSIGNED') && list.length >= 65535) {
          throw new Error('id超过类型')
        }
      } else if (idType.includes('MEDIUMINT')) {
        if (idType.includes('UNSIGNED') && list.length >= 8388607) {
          throw new Error('id超过类型')
        } else if (!idType.includes('UNSIGNED') && list.length >= 16777215) {
          throw new Error('id超过类型')
        }
      }

      await app.model[model].sync({
        force: true,
      })

      await app.model[model].bulkCreate(list)
    } catch (e) {
      console.log(`${model}出错,${e.message}`)
      continue
    }
  }

  ctx.SUCCESS('ok')
}

exports.verify = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)
  const { code } = ctx.request.body
  const verify = util.promisify(jwt.verify)
  const result = await verify(code, 'token')
  ctx.SUCCESS(result)
}
