const {
  getAppByCtx,
  getConfig,
  lodash,
  moment,
  getService,
} = require('multiple-quick-koa')
const util = require('util')
const axios = require('axios')
const qr = require('qr-image')

const getCodeUrl =
  'https://open.douyin.com/platform/oauth/connect?client_key=%s&redirect_uri=%s&response_type=code&scope=%s&state=%s'

const Douyin = getService('douyin')
const LRU = require('lru-cache')
const cache = new LRU({
  max: 100,
  maxAge: 1000 * 60 * 60,
})

/**
 * @apiRoute login_callback
 * @apiName 登录回调
 * @apiParam code/string/code
 * @apiParam state/string/user_id
 * @apiReturn {null}
 */
exports.login_callback = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { code, state: user_id } = ctx.query
  if (!code) {
    return ctx.redirect(
      'https://doudou.kuashou.com/doudou/#/pages/accountFail/accountFail'
    )
  }

  const appConfig = getConfig(app)
  const { client_key, client_secret } = await appConfig.getObject('douyin')
  const url = `https://open.douyin.com/oauth/access_token/?client_key=${client_key}&client_secret=${client_secret}&code=${code}&grant_type=authorization_code`

  const codeResult = await axios.get(url).then((res) => res.data)

  if (codeResult.message !== 'success') {
    throw new Error(codeResult.data.description)
  }
  const { open_id, access_token } = codeResult.data
  if (!open_id) throw new Error('抖音授权登录失败')

  const checkExist = await app.model.douyin_user.findOne({
    where: {
      openid: open_id,
    },
    include: [
      {
        model: app.model.user,
        include: app.model.h5_user,
      },
    ],
  })

  const errorObj = {
    hasOther: checkExist
      ? {
          msg: '该抖音号已绑定过其他账户',
          user: {
            nick_name: checkExist.user.nick_name,
            image: checkExist.user.image,
          },
          change_user_id: Number(user_id),
          self_id: checkExist.id,
        }
      : null,
  }

  if (checkExist && checkExist.user_id !== Number(user_id)) {
    return ctx.redirect(
      `https://doudou.kuashou.com/doudou/#/pages/accountFail/accountFail?error=${encodeURIComponent(
        JSON.stringify(errorObj.hasOther)
      )}`
    )
  }

  const userInfoUrl = `https://open.douyin.com/oauth/userinfo/?open_id=${open_id}&access_token=${access_token}`
  const userInfoResult = await axios.get(userInfoUrl).then((res) => res.data)

  if (userInfoResult.message !== 'success') {
    throw new Error(userInfoResult.data.description)
  }

  // app.model.douyin_user.upsert({
  //   ...userInfoResult.data,
  //   openid: userInfoResult.data.open_id,
  //   nick_name: userInfoResult.data.nickname,
  // })
  const rest = userInfoResult.data
  const post = lodash.pickBy(
    {
      nick_name: lodash.get(rest, 'nickname', null),
      avatar: lodash.get(rest, 'avatar', null),
      country: lodash.get(rest, 'country', null),
      city: lodash.get(rest, 'city', null),
      province: lodash.get(rest, 'province', null),
      gender: lodash.get(rest, 'gender', null),
      openid: rest.open_id,
      user_id: user_id ? Number(user_id) : null,
      access_token: access_token || '',
    },
    (item) => item !== null
  )

  const openIdUser = await app.model.douyin_user.findOne({
    where: {
      openid: post.openid,
    },
  })

  let uid
  if (openIdUser) {
    await app.model.douyin_user.update(post, {
      where: {
        openid: post.openid,
      },
    })
    uid = openIdUser.user_id
  } else {
    if (post.user_id) {
      uid = post.user_id
    } else {
      const user = await app.model.user.create()
      uid = user.id
    }

    await app.model.douyin_user.create({
      ...post,
      user_id: uid,
    })
  }

  const result = await app.model.user.findOne({
    where: {
      id: uid,
    },
    include: [
      {
        model: app.model.h5_user,
      },
      {
        model: app.model.douyin_user,
      },
      ...app.include.user,
    ],
  })

  const tokenResult = {
    id: result.id,
    name: result.name,
    mobile: result.mobile,
    h5_user: result.h5_user
      ? {
          openid: result.h5_user && result.h5_user.openid,
        }
      : null,
    douyin_users: result.douyin_users
      ? result.douyin_users.map((item) => {
          return {
            openid: item && item.openid,
          }
        })
      : [],
  }

  const token = await app.sign({
    user: tokenResult,
  })

  // return ctx.SUCCESS({
  //   token,
  //   user: result
  // })

  return ctx.redirect(
    'https://doudou.kuashou.com/doudou/#/pages/accountSuccess/accountSuccess'
  )
}

exports.createImg = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)

  const appConfig = getConfig(app)
  const { client_key, host_link } = await appConfig.getObject('douyin')

  const redirect_uri = `${host_link}/doudou/douyin/login_callback`
  // const client_key = 'awrygfkjh7tts5rh'
  // const scope = 'aweme.share,user_info,micapp.is_legal,video.create,video.delete,video.data,video.list,fans.data'

  const scope =
    'aweme.share,user_info,micapp.is_legal,video.create,video.delete,video.data,video.list,fans.data'
  const url = util.format(
    getCodeUrl,
    client_key,
    redirect_uri,
    scope,
    ctx.query.user_id ? ctx.query.user_id : null
  )
  // const result = await axios.get(url, {
  //   client_key: app_id,
  // }).then(res => res.data)
  console.log('失败先检查client_key')
  const img = qr.image(url, {
    size: 2,
  })
  ctx.type = 'image/png'
  ctx.body = img

  // ctx.SUCCESS(result)
}

exports.fansData = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { id } = ctx.request.body

  const result = await app.model.douyin_user.findOne({
    where: {
      id,
    },
  })

  if (!result) throw new Error('找不到用户')

  const url = 'https://open.douyin.com/fans/data/?open_id=%s&access_token=%s'

  const videoResult = await axios
    .get(util.format(url, result.openid, result.access_token))
    .then((res) => res.data)
  if (videoResult.data.error_code) {
    if (videoResult.data.error_code === 2190008) {
      throw new Error('请重新授权')
    }
    throw new Error(videoResult.data.description)
  }
  ctx.SUCCESS(videoResult.data.fans_data)
}

exports.videoList = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { id } = ctx.request.body

  const result = await app.model.douyin_user.findOne({
    where: {
      id,
    },
  })

  if (!result) throw new Error('找不到用户')

  const url =
    'https://open.douyin.com/video/list/?open_id=%s&access_token=%s&count=%s'

  const videoResult = await axios
    .get(util.format(url, result.openid, result.access_token, 10))
    .then((res) => res.data)
  if (videoResult.data.error_code) {
    if (videoResult.data.error_code === 2190008) {
      throw new Error('请重新授权')
    }
    throw new Error(videoResult.data.description)
  }
  ctx.SUCCESS(videoResult.data.list)
}

const updateTask = async ({ app, list, active_time, douyin }) => {
  if (list && list.length > 0) {
    await Promise.all(
      ['hot_search', 'hot_video'].map((item) => {
        return app.model[item].sync({
          force: true,
        })
      })
    )

    await app.model.hot_search.bulkCreate(
      list.map((item) => {
        const { sentence, hot_level } = item
        return {
          name: sentence,
          level: hot_level,
          created_at: active_time,
          updated_at: active_time,
        }
      })
    )

    const newList = await app.model.hot_search.findAll({
      limit: 3,
      order: [['level', 'DESC']],
    })
    for (let i = 0; i < newList.length; i++) {
      const { id, name } = newList[i]
      // const {
      //   id
      // } = await app.model.hot_search.create({
      //   name: sentence,
      //   level: hot_level,
      //   created_at: active_time,
      //   updated_at: active_time
      // })
      // console.log('生成' + id)
      let videoItemList = []
      if (i < 3) {
        try {
          const { list: videoList } = await douyin.getHotVideo(name)
          videoItemList = [
            ...videoItemList,
            ...videoList.map((video) => {
              const {
                is_top,
                share_url,
                statistics,
                create_time,
                is_reviewed,
                title,
                video_status,
                cover,
                item_id,
              } = video

              const status = {
                1: '已发布',
                2: '不适宜公开',
                4: '审核中',
              }
              return {
                hot_search_id: id,
                image: cover,
                is_top,
                share_url,
                play_count: statistics.play_count,
                share_count: statistics.share_count,
                comment_count: statistics.comment_count,
                digg_count: statistics.digg_count,
                download_count: statistics.download_count,
                forward_count: statistics.forward_count,
                created_at: moment(create_time * 1000),
                is_reviewed,
                name: title,
                video_status: status[video_status],
                itemid: item_id,
              }
            }),
          ]
        } catch (e) {
          continue
        }
      }

      await app.model.hot_video.bulkCreate(videoItemList)
    }
  }
}
exports.hotSearch = async (ctx) => {
  const { app } = getAppByCtx(ctx)
  console.log('33', 33)

  const hotWord = await app.model.hot_search.findOne()
  const minutes = hotWord
    ? moment().diff(moment(hotWord.created_at), 'minutes')
    : 1000

  if (minutes < 30) {
    console.log('离上次更新', minutes)
    const result = await app.model.hot_search.findAll({
      order: [['level', 'DESC']],
      include: [
        {
          model: app.model.hot_video,
          limit: 1,
          where: {
            image: {
              $ne: '',
            },
          },
        },
      ],
    })
    ctx.SUCCESS(result)
    return result
  }

  const appConfig = getConfig(app)

  const { client_key, client_secret } = await appConfig.getObject('douyin')
  const douyin = new Douyin({
    client_key,
    client_secret,
  })

  try {
    const { active_time, list } = await douyin.getHotSearch()
    updateTask({
      app,
      list,
      active_time,
      douyin,
    })

    const result = list.map((item) => {
      return {
        ...item,
        name: item.sentence,
        level: item.hot_level,
        hot_videos: [],
        created_at: active_time,
        updated_at: active_time,
      }
    })
    ctx.SUCCESS(result)
    return result
  } catch (e) {
    console.log('ee')
    const result = await app.model.hot_search.findAll({
      order: [['level', 'DESC']],
      include: [
        {
          model: app.model.hot_video,
          limit: 1,
          where: {
            image: {
              $ne: '',
            },
          },
        },
      ],
    })

    ctx.SUCCESS(result)
    return result
  }
}

exports.hotSearchTrending = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  if (cache.get('hotSearchTrending')) {
    return ctx.SUCCESS(cache.get('hotSearchTrending'))
  }
  const appConfig = getConfig(app)
  const { client_key, client_secret } = await appConfig.getObject('douyin')
  const douyin = new Douyin({
    client_key,
    client_secret,
  })

  try {
    const result = await douyin.getHotSearchTrending()
    cache.set('hotSearchTrending', result)
    ctx.SUCCESS(result)
  } catch (e) {
    throw new Error(e.message)
  }
}

exports.hotVideo = async (ctx) => {
  const { app } = getAppByCtx(ctx)
  const { word } = ctx.request.body
  if (cache.get(word)) {
    return ctx.SUCCESS(cache.get(word))
  }

  const appConfig = getConfig(app)
  const { client_key, client_secret } = await appConfig.getObject('douyin')

  const douyin = new Douyin({
    client_key,
    client_secret,
  })
  try {
    const result = await douyin.getHotVideo(word)
    cache.set(word, result)
    ctx.SUCCESS(result)
  } catch (e) {
    throw new Error(e.message)
  }
}

exports.starHot = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const {
    // 达人热榜类型 * `1` - 星图指数榜 * `2` - 涨粉指数榜 * `3` - 性价比指数榜 * `4` - 种草指数榜 * `5` - 精选指数榜 * `6` - 传播指数榜
    type = 1,
  } = ctx.request.body
  if (cache.get(`starHot-${type}`)) {
    return ctx.SUCCESS(cache.get(`starHot-${type}`))
  }
  const appConfig = getConfig(app)
  const { client_key, client_secret } = await appConfig.getObject('douyin')
  console.log(client_key, 'client_key')
  const douyin = new Douyin({
    client_key,
    client_secret,
  })

  const douyin1 = new Douyin({
    client_key,
    client_secret,
  })

  console.log('true', douyin1 === douyin)
  try {
    const result = await douyin.getStarHot(type)

    cache.set(`starHot-${type}`, result)
    ctx.SUCCESS(result)
  } catch (e) {
    throw new Error(e.message)
  }
}
