const util = require('util')
const axios = require('axios')
const { lodash } = require('multiple-quick-koa')

const commonTokenUrl =
  'https://open.douyin.com/oauth/client_token/?client_key=%s&client_secret=%s&grant_type=client_credential'
const hotSearchUrl =
  'https://open.douyin.com/hotsearch/sentences/?access_token=%s'
const hotSearchTrendingUrl =
  'https://open.douyin.com/hotsearch/trending/sentences/?access_token=%s&cursor=%s&count=%s'
const hotVideoUrl =
  'https://open.douyin.com/hotsearch/videos/?access_token=%s&hot_sentence=%s&dateType=%s'
const starHostListUrl =
  'https://open.douyin.com/star/hot_list/?access_token=%s&hot_list_type=%s'

const LRU = require('lru-cache')
const cache = new LRU({
  max: 100,
  maxAge: 1000 * 60 * 60,
})

module.exports = class Singleton {
  constructor(config) {
    this.config = {
      ...config,
    }

    /**
     * 单例模式
     */
    if (!Singleton.instance) {
      Singleton.instance = this
    }
    const previous = Singleton.instance.getConfig()
    if (!lodash.isEqual(previous, config)) {
      Singleton.instance = this
    }
    return Singleton.instance
  }

  init() {
    if (!this.config.client_key || !this.config.client_secret) {
      throw new Error('没有配置client_key或client_secret')
    }
  }

  async getAccessToken() {
    const { client_key, client_secret } = this.config

    if (cache.get(client_key)) {
      return cache.get(client_key)
    }
    const url = util.format(commonTokenUrl, client_key, client_secret)
    const tokenResult = await axios.get(url).then((res) => res.data)
    if (tokenResult.message !== 'success') {
      throw new Error(tokenResult.data.description)
    }
    cache.set(client_key, tokenResult.data.access_token)
    return tokenResult.data.access_token
  }

  async getHotSearch() {
    const access_token = await this.getAccessToken()
    try {
      const hotsearchResult = await axios
        .get(util.format(hotSearchUrl, access_token))
        .then((res) => res.data)
      if (hotsearchResult.data.error_code) {
        if (hotsearchResult.data.error_code === 2190008) {
          cache.reset()
          return await this.getHotSearch()
        }
        throw new Error(hotsearchResult.data.description)
      }

      return hotsearchResult.data
    } catch (e) {
      console.log(e)
      throw new Error('请求抖音失败')
    }
  }

  async getHotSearchTrending(offset, limit) {
    const access_token = await this.getAccessToken()
    offset = offset || 0
    limit = limit || 20
    try {
      const hotSearchTrendingResult = await axios
        .get(util.format(hotSearchTrendingUrl, access_token, offset, limit))
        .then((res) => res.data)
      if (hotSearchTrendingResult.data.error_code) {
        if (hotSearchTrendingResult.data.error_code === 2190008) {
          cache.reset()
          return await this.getHotSearchTrending(offset, limit)
        }
        throw new Error(hotSearchTrendingResult.data.description)
      }

      return hotSearchTrendingResult.data
    } catch (e) {
      console.log(e)
      throw new Error('请求抖音失败')
    }
  }

  async getHotVideo(word, day = 7) {
    if (!word) throw new Error('没有热点词')
    const access_token = await this.getAccessToken()
    const url = util.format(hotVideoUrl, access_token, encodeURI(word), day)
    try {
      const hotVideoResult = await axios.get(url).then((res) => res.data)

      if (hotVideoResult.data.error_code) {
        if (hotVideoResult.data.error_code === 2190008) {
          cache.reset()
          return await this.getHotVideo(word, day)
        }
        throw new Error(hotVideoResult.data.description)
      }
      if (!hotVideoResult.data.list) throw new Error(`${word}不是热词`)
      return hotVideoResult.data
    } catch (e) {
      console.log(e)
      throw new Error('请求抖音失败')
    }
  }

  async getStarHot(hot_list_type) {
    const access_token = await this.getAccessToken()
    const url = util.format(starHostListUrl, access_token, hot_list_type)
    try {
      const result = await axios.get(url).then((res) => res.data)

      if (result.data.error_code) {
        if (result.data.error_code === 2190008) {
          cache.reset()
          return await this.getStarHot(hot_list_type)
        }
        throw new Error(result.data.description)
      }
      return result.data
    } catch (e) {
      console.log(e)
      throw new Error('请求抖音失败')
    }
  }

  getConfig() {
    return this.config
  }
}
