const util = require('util')
const axios = require('axios')

const { lodash } = require('multiple-quick-koa')
const getAccessTokenUrl =
  'https://developer.toutiao.com/api/apps/token?grant_type=%s&appid=%s&secret=%s'
const createQRCodeUrl = 'https://developer.toutiao.com/api/apps/qrcode'

const fs = require('fs')
const LRU = require('lru-cache')
const cache = new LRU({
  max: 100,
  maxAge: 1000 * 60 * 60,
})
const uuid = require('node-uuid')

module.exports = class Singleton {
  constructor(config) {
    this.config = {
      ...config,
      grant_type: 'client_credential',
    }

    /**
     * 单例模式
     */
    if (!Singleton.instance) {
      Singleton.instance = this
    }
    const previous = Singleton.instance.getConfig()
    if (!lodash.isEqual(previous, config)) {
      Singleton.instance = this
    }
    return Singleton.instance
  }

  init() {
    if (!this.config.appid || !this.config.secrect)
      throw new Error('没有配置appid或secrect')
  }

  async getAccessToken() {
    const { appid, secrect, grant_type } = this.config
    if (cache.get(appid)) {
      return cache.get(appid)
    }
    const url = util.format(getAccessTokenUrl, grant_type, appid, secrect)
    const { access_token } = await axios.get(url).then((res) => res.data)
    cache.set(appid, access_token)
    return access_token
  }

  async createQR(options) {
    if (!this.config.targetPath) throw new Error('没有配置文件夹')

    const isExists = fs.existsSync(
      `${this.config.targetPath}/${options.filename}.png`
    )
    if (isExists) return `${options.filename}.png`

    const access_token = await this.getAccessToken()
    const payLoad = {
      appname: 'douyin',
      filename: '',
      access_token,
      ...options,
    }
    const creatuuid = payLoad.filename ? payLoad.filename : uuid.v1()

    const bufferResult = await axios
      .post(createQRCodeUrl, payLoad, {
        headers: {
          'content-type': 'application/json',
        },
        responseType: 'arraybuffer',
      })
      .then((res) => {
        if (res.headers['content-type'].includes('json')) {
          return JSON.parse(Buffer.from(res.data).toString('utf8'))
        }
        return res.data
      })

    if (bufferResult.errcode) {
      if (bufferResult.errcode === 40001) {
        cache.reset()
        return await this.createQR(options)
      }
      throw new Error(bufferResult.errmsg)
    }
    await fs.writeFileSync(
      `${this.config.targetPath}/${creatuuid}.png`,
      bufferResult
    )

    return `${creatuuid}.png`
  }

  getConfig() {
    return this.config
  }
}
