const util = require('util')
const axios = require('axios')

const { lodash } = require('multiple-quick-koa')
const getAccessTokenUrl =
  'https://api.weixin.qq.com/cgi-bin/token?grant_type=%s&appid=%s&secret=%s'
const createQRCodeUrl =
  'https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token=%s'
const sendMessageUrl =
  'https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token=%s'
const handleMessageUrl =
  'https://api.weixin.qq.com/cgi-bin/message/custom/send?access_token=%s'
const uploadFile =
  'https://api.weixin.qq.com/cgi-bin/media/upload?access_token=%s&type=%s'
const serviceMarketUrl =
  'https://api.weixin.qq.com/wxa/servicemarket?access_token=%s&type=%s'
const createLiveUrl =
  'https://api.weixin.qq.com/wxaapi/broadcast/room/create?access_token=%s'
const getLiveUrl =
  'https://api.weixin.qq.com/wxa/business/getliveinfo?access_token=%s'
const addSubanchorUrl =
  'https://api.weixin.qq.com/wxaapi/broadcast/room/addsubanchor?access_token=%s'

const addRoleUrl =
  'https://api.weixin.qq.com/wxaapi/broadcast/addRole?access_token=%s'

const getExpressUrl =
  'https://api.weixin.qq.com/cgi-bin/express/business/delivery/getall?access_token=%s'
const addOrderUrl =
  'https://api.weixin.qq.com/cgi-bin/express/business/order/add?access_token=%s'

const path = require('path')
const fs = require('fs')
const LRU = require('lru-cache')
const request = require('request')
const cache = new LRU({
  max: 100,
  maxAge: 1000 * 60 * 60,
})
const uuid = require('node-uuid')

const uploadPromise = (options) => {
  return new Promise((resolve, reject) => {
    request.post(
      {
        ...options,
      },
      (err, response, body) => {
        if (err || response.statusCode !== 200) {
          reject(err)
        }
        resolve(JSON.parse(body))
      }
    )
  })
}

module.exports = class Singleton {
  constructor(config) {
    this.config = {
      ...config,
      grant_type: 'client_credential',
    }
    /**
     * 单例模式
     */
    if (!Singleton.instance) {
      Singleton.instance = this
    }
    const previous = Singleton.instance.getConfig()
    if (!lodash.isEqual(previous, config)) {
      Singleton.instance = this
    }
    return Singleton.instance
  }

  init() {
    if (!this.config.appid || !this.config.secrect) {
      throw new Error('没有配置appid或secrect')
    }
  }

  async getAccessToken() {
    const { appid, secrect, grant_type } = this.config
    if (cache.get(appid)) {
      return cache.get(appid)
    }
    const url = util.format(getAccessTokenUrl, grant_type, appid, secrect)
    const { access_token } = await axios.get(url).then((res) => res.data)
    cache.set(appid, access_token)
    return access_token
  }

  async getExpress() {
    const access_token = await this.getAccessToken()
    const url = util.format(getExpressUrl, access_token)
    const result = await axios.post(url).then((res) => res.data)
    if (result.errcode) {
      if (result.errcode === 40001) {
        cache.reset()
        return await this.getExpress()
      }
      throw new Error(`${result.errcode};${result.errmsg}`)
    }
    return result
  }

  async addOrder({
    add_source = 0,
    order_id,
    openid = '',
    delivery_id = '',
    biz_id = '',
    sender,
    receiver,
    cargo,
    shop,
    insured,
    service,
  }) {
    const access_token = await this.getAccessToken()
    const url = util.format(addOrderUrl, access_token)

    const formatOptions = {
      order_id,
      openid,
      delivery_id,
      biz_id,
      sender,
      receiver,
      cargo,
      shop,
      insured,
      service,
    }

    const result = await axios.post(url, formatOptions).then((res) => res.data)
    if (result.errcode) {
      if (result.errcode === 40001) {
        cache.reset()
        return await this.addOrder({
          add_source,
          order_id,
          openid,
          delivery_id,
          biz_id,
          sender,
          receiver,
          cargo,
          shop,
          insured,
          service,
        })
      }
      throw new Error(`${result.errcode};${result.errmsg}`)
    }
    return result
  }

  async addSubanchor({ roomId, username }) {
    const access_token = await this.getAccessToken()
    const url = util.format(addSubanchorUrl, access_token)

    const formatOptions = {
      roomId,
      username,
    }

    const result = await axios.post(url, formatOptions).then((res) => res.data)
    if (result.errcode) {
      if (result.errcode === 40001) {
        cache.reset()
        return await this.addSubanchor({
          roomId,
          username,
        })
      }
      throw new Error(`${result.errcode};${result.errmsg}`)
    }
    return result
  }

  async addRole({ username, role = 2 }) {
    const access_token = await this.getAccessToken()
    const url = util.format(addRoleUrl, access_token)

    const formatOptions = {
      username,
      role,
    }
    const result = await axios.post(url, formatOptions).then((res) => res.data)
    if (result.errcode) {
      if (result.errcode === 40001) {
        cache.reset()
        return await this.addRole({
          username,
          role,
        })
      }
      throw new Error(`${result.errcode};${result.errmsg}`)
    }
    return result
  }

  async createLive({
    name,
    createrWechat,
    coverImg,
    startTime,
    endTime,
    shareImg,
    feedsImg,
    subAnchorWechat = '',
    anchorWechat = '',
    anchorName = '',
    closeReplay = 1,
  }) {
    const access_token = await this.getAccessToken()
    const url = util.format(createLiveUrl, access_token)

    const formatOptions = {
      name, // 房间名字
      coverImg, // 通过 uploadfile 上传，填写 mediaID
      startTime, // 开始时间
      endTime, // 结束时间
      anchorName, // 主播昵称
      anchorWechat, // 主播微信号
      subAnchorWechat, // 主播副号微信号
      createrWechat, // 创建者微信号
      shareImg, // 通过 uploadfile 上传，填写 mediaID
      feedsImg, // 通过 uploadfile 上传，填写 mediaID
      isFeedsPublic: 0, // 是否开启官方收录，1 开启，0 关闭
      type: 0, // 直播类型，1 推流 0 手机直播
      closeLike: 0, // 是否关闭点赞 1：关闭
      closeGoods: 1, // 是否关闭商品货架，1：关闭
      closeComment: 1, // 是否开启评论，1：关闭
      closeReplay, // 是否关闭回放 1 关闭
      closeShare: 1, //  是否关闭分享 1 关闭
      closeKf: 1, // 是否关闭客服，1 关闭
    }
    const result = await axios.post(url, formatOptions).then((res) => res.data)
    if (result.errcode) {
      if (result.errcode === 40001) {
        cache.reset()
        return await this.createLive({
          name,
          createrWechat,
          coverImg,
          startTime,
          endTime,
          shareImg,
          feedsImg,
          subAnchorWechat,
          anchorWechat,
          anchorName,
        })
      }
      throw new Error(`${result.errcode};${result.errmsg}`)
    }
    return result
  }

  async getLive(start = 0, limit = 10) {
    const access_token = await this.getAccessToken()
    const url = util.format(getLiveUrl, access_token)

    const formatOptions = {
      start,
      limit,
    }
    const result = await axios.post(url, formatOptions).then((res) => res.data)
    if (result.errcode) {
      if (result.errcode === 40001) {
        cache.reset()
        return await this.getLive({
          start,
          limit,
        })
      }
      throw new Error(`${result.errcode};${result.errmsg}`)
    }
    return result.room_info
  }

  async createQR(options) {
    if (!this.config.targetPath) throw new Error('没有配置文件夹')
    const access_token = await this.getAccessToken()
    const url = util.format(createQRCodeUrl, access_token)
    const payLoad = {
      ...options,
      scene: options.scene
        ? Object.keys(options.scene)
            .map((item) => `${item}=${options.scene[item]}`)
            .join('&')
        : '',
    }
    const creatuuid = uuid.v1()

    const bufferResult = await axios
      .post(url, payLoad, {
        responseType: 'arraybuffer',
      })
      .then((res) => {
        if (res.headers['content-type'].includes('json')) {
          return JSON.parse(Buffer.from(res.data).toString('utf8'))
        }
        return res.data
      })
    if (bufferResult.errcode) {
      if (bufferResult.errcode === 40001) {
        cache.reset()
        return await this.createQR(options)
      }
      throw new Error(bufferResult.errmsg)
    }
    await fs.writeFileSync(
      `${this.config.targetPath}/${creatuuid}.png`,
      bufferResult
    )
    return `${creatuuid}.png`
  }

  async createQRBuffer(options) {
    const access_token = await this.getAccessToken()
    const url = util.format(createQRCodeUrl, access_token)
    const payLoad = {
      ...options,
      scene: options.scene
        ? Object.keys(options.scene)
            .map((item) => `${item}=${options.scene[item]}`)
            .join('&')
        : '',
    }
    const creatuuid = uuid.v1()

    const bufferResult = await axios
      .post(url, payLoad, {
        responseType: 'arraybuffer',
      })
      .then((res) => {
        if (res.headers['content-type'].includes('json')) {
          return JSON.parse(Buffer.from(res.data).toString('utf8'))
        }
        return res.data
      })
    if (bufferResult.errcode) {
      if (bufferResult.errcode === 40001) {
        cache.reset()
        return await this.createQR(options)
      }
      throw new Error(bufferResult.errmsg)
    }
    return bufferResult
  }

  async sendMessage(options) {
    const access_token = await this.getAccessToken()
    const url = util.format(sendMessageUrl, access_token)
    const formatOptions = {
      ...options,
      data: lodash.mapValues(options.data, (item) => {
        return {
          value: item,
        }
      }),
    }
    const result = await axios.post(url, formatOptions).then((res) => res.data)
    if (result.errcode) {
      if (result.errcode === 40001) {
        cache.reset()
        return await this.sendMessage(options)
      }
      throw new Error(`${result.errcode};${result.errmsg}`)
    }
    return result
  }

  async handleMessage(options) {
    const access_token = await this.getAccessToken()
    const url = util.format(handleMessageUrl, access_token)
    const result = await axios.post(url, options).then((res) => res.data)
    if (result.errcode) {
      if (result.errcode === 40001) {
        cache.reset()
        return await this.handleMessage(options)
      }
      throw new Error(`${result.errcode};${result.errmsg}`)
    }
    return result
  }

  async uploadFile(imgUrl) {
    if (!imgUrl || !imgUrl.includes('//')) throw new Error('检查imgUrl是否链接')

    const access_token = await this.getAccessToken()
    const url = util.format(uploadFile, access_token, 'image')
    const formData = {
      media: request(imgUrl.startsWith('//') ? `http:${imgUrl}` : imgUrl),
    }
    const uploadResult = await uploadPromise({
      url,
      formData,
    })
    if (uploadResult.errcode) {
      if (uploadResult.errcode === 40001) {
        cache.reset()
        return await this.uploadFile(imgUrl)
      }
      throw new Error(`${uploadResult.errcode};${uploadResult.errmsg}`)
    }
    return uploadResult.media_id
  }

  async invokeService(payLoad) {
    const access_token = await this.getAccessToken()
    const url = util.format(serviceMarketUrl, access_token)
    const result = await axios
      .post(url, {
        ...payLoad,
      })
      .then((res) => res.data)
    if (result.errcode) {
      if (result.errcode === 40001) {
        cache.reset()
        return await this.invokeService(payLoad)
      }
      throw new Error(`${result.errcode};${result.errmsg}`)
    }
    return result.data
  }

  getConfig() {
    return this.config
  }
}
